#include "curvtorscalculator.h"
#include "streamlinepool.h"

#include "vector_types.h"
#include <algorithm>

#define PI 3.1415927f

void CurvTorsCalculator::computeDiscreteCurvTors()
{

    for(auto s : _pStreamlines->streamlines) {
        Attribute<float3>* points = dynamic_cast<Attribute<float3>*>((s->attributes)["points"]);

        Attribute<float>* curv = new Attribute<float>(s->numPts);    
        Attribute<float>* tors = new Attribute<float>(s->numPts-1);    
        (s->attributes)["curvature"] = curv;
        (s->attributes)["torsion"] = tors;

        float3 p1 = (*points)[0];
        float3 p2 = (*points)[1];
        float3 p3 = (*points)[2];
        float3 p4;
        float3 d1 = normalize(p2-p1);
        float3 d2 = normalize(p3-p2);
        float3 d3;
        for(unsigned i = 1; i < s->numPts - 2; i++) {

            p4 = (*points)[i+2];

            d3 = normalize(p4 - p3);

            // construct the local frames
            float3 X1 = d1;
            float3 X2 = d2;
            float3 Y1 = make_float3(0, 0, 0);
            float3 Y2 = make_float3(0, 0, 0);
            if(length(d2 - dot(d1,d2) * d1) > 1e-6)
                Y1 = normalize(d2 - dot(d1, d2) * d1);

            if(length((d3 - dot(d3, d2) * d2)) > 1e-6)
                Y2 = normalize(d3 - dot(d3, d2) * d2);

            // compute curvatue and torsion
            if( dot(X1, X2) >= 1.0 )
                (*curv)[i] = 0;
            else
                (*curv)[i] = acos( dot(X1, X2) );

            if( length(Y1) == 0 || length(Y2) == 0 )
                (*tors)[i] = .0f;
            else {
                float3 Z1 = normalize(cross(X1, Y1));
                float3 Z2 = normalize(cross(X2, Y2));

                if( dot(Z1, d3) >= 0 ) {
                    if( fabs(dot(Z1, Z2)) >= 1.0 )
                        (*tors)[i] = .0f;
                    else {
                        (*tors)[i] = acos(dot(Z1,Z2));
                    }
                }
                else {
                    if( fabs(dot(Z1, Z2)) >= 1.0 )
                        (*tors)[i] = .0f;
                    else {
                        (*tors)[i] = -acos(dot(Z1,Z2));
                    }
                }
            }

			(*tors)[i] +=  PI;
			(*tors)[i] = std::max((*tors)[i], .0f);
			(*tors)[i] = std::min((*tors)[i], 2.0f*PI);

            p3 = p4;

            d1 = d2;
            d2 = d3;

        }

        // curvature of the second to last point
        if( dot(d1, d2) >= 1.0 )
            (*curv)[s->numPts-2] = .0f;
        else
            (*curv)[s->numPts-2] = acos( dot(d1, d2) );

		// set curv to 0 for the first and the last point
		(*curv)[0] = (*curv)[s->numPts - 1] = .0f;

		// set tors to 0 for the first and the last segment
		(*tors)[0] = (*tors)[s->numPts - 2] = .0f;

    }

}

