#include "streamlinepool.h"
#include "curvtorscalculator.h"
#include "entropycalculator.h"
#include "minvolenclosingellipsoid.h"
#include "trainingstreamlinepicker.h"
#include "trainingdata.h"
#include "streamlinesimplifier.h"
#include "segmenter.h"
#include "segmenthistcalculator.h"

int main(int argc, char** argv) {

	// read the input vector field and trace 100 streamlines
	StreamlinePool* stl_pool = new StreamlinePool();
	stl_pool->readVelocityField("your_vector_field.vel");
	stl_pool->trace(100, false);
	
	// pass streamlines around
	TortuosityCalculator::instance().setStreamlinePool(stl_pool);
	EntropyCalculator::instance().setStreamlinePool(stl_pool);
	SegmentHistogramCalculator::instance().setStreamlinePool(stl_pool);
	CurvTorsCalculator::instance().setStreamlinePool(stl_pool);
	MVEE::instance().setStreamlinePool(stl_pool);
	Trainer::instance().setStreamlinePool(stl_pool);
	Segmenter::instance().setStreamlinePool(stl_pool);
	
	// generate representative streamlines for training
	TrainingStreamlinePicker* stl_picker = new TrainingStreamlinePicker();
	stl_picker->setStreamlines(stl_pool->streamlines);
	
	// simplify streamlines
	StreamlineSimplifier::instance().simplify(stl_pool->streamllines);
	
	// training
	// here we go through all the representative streamlines; but in practice, you don't have to do this
	int sid;
	while( (sid = stl_picker->pickStreamline()) != -1 ) {
		// you need to let trainer know the current streamline sid
		
		// the example here shows that point 5 on sid is added as a positive example, and point 10 is added as a negative example
		// however, as described in the paper, negative training examples are automatically picked 
		Trainer::instance().add_seg_point_example(5, true);
		Trainer::instance().add_seg_point_example(10 ,false);
	}
	
	// start training
	Trainer::instance().train();
	
	// segment all the streamlines
	Segmenter::instance().segment();
	
	return 0;
	

}