#ifndef MINVOLENCLOSINGELLIPSOID_H
#define MINVOLENCLOSINGELLIPSOID_H

#include <Eigen/Dense>
#include <memory>
#include <vector>
#include "vector_types.h"

using namespace std;

class StreamlinePool;

struct Ellipsoid {
	Eigen::Matrix3f axes;
	Eigen::Vector3f radii;

	Eigen::Matrix3f Q;
	Eigen::Vector3f c;


};

class MVEE {
public:
	static MVEE& instance() {
		static MVEE inst;
		return inst;
	}

	// compute volume ratio between two min volume bounding ellipsoids of the points in [center-nhs, nhs] and [nhs, nhs+center]  
	float computeBoundingEllipsoidVolRatio(int sid, int center, int nhs);

	float computeBoundingEllipsoidVolRatio(int sid, int start, int mid, int end);

	void setStreamlinePool(shared_ptr<StreamlinePool> pool) { _pStreamlines = pool;  }

private:
	MVEE() {}

	float computeEllipsoidVolume(const vector<float3>& points, int start, int end);
	bool boundingEllipsoid(const Eigen::Matrix3Xf& P, Eigen::Matrix3f& Q, Eigen::Vector3f& c, float tolerence = 0.1);
	void computeEllipsoidParameters(Ellipsoid&);
	float ellipsoidVolume(const Eigen::Vector3f& radii);

private:
	shared_ptr<StreamlinePool>	_pStreamlines;
};

#endif