#include "segmenter.h"
#include "streamlinepool.h"
#include "segmenthistcalculator.h"
#include "trainingdata.h"
#include "entropycalculator.h"
#include "curvtorscalculator.h"
#include "tortuositycalculator.h"
#include "minvolenclosingellipsoid.h"



#include <cassert>
#include <queue>
#include <algorithm>
#include <iostream>


void Segmenter::segment()
{
	for (size_t sid = 0; sid < _pStreamlines->streamlines.size(); sid++) {
		Streamline* s = _pStreamlines->streamlines[sid];
		assert(s->attributes.find("simplification_points") != s->attributes.end());
		Attribute<unsigned> *simplification_points = dynamic_cast<Attribute<unsigned>*>(s->attributes["simplification_points"]);

		Attribute<unsigned> *segPoints;
		if (s->attributes.find("segmentation_points") == s->attributes.end()){
			segPoints = new Attribute<unsigned>;
			s->attributes["segmentation_points"] = segPoints;
		}
		else {
			segPoints = dynamic_cast<Attribute<unsigned>*>(s->attributes["segmentation_points"]);
			segPoints->clear();
		}


		float margin;
		for (int i = 1; i < int((*simplification_points).size()) - 1; i++) {
			int id = (*simplification_points)[i];
			if (is_seg_point(sid, id, margin)) 
				segPoints->push_back(id);	
		}



		if (segPoints->size() > 1 && _cleanup )
			clean_up_segmentation(sid, segPoints);
	}

}

bool Segmenter::is_seg_point(int sid, int id, float& margin)
{
	vector<float> features;
	Trainer::instance().compute_point_features(sid, id, features);
	svm_node* x = new svm_node[features.size() + 1];
	size_t i = 0;
	for (; i < features.size(); i++) {
		x[i].index = i + 1;
		x[i].value = features[i];
	}
	x[i].index = -1;

	int label = svm_predict(Trainer::instance().get_seg_point_classifier(), x);
	delete[] x;

	return label == 1;
}




void Segmenter::clean_up_segmentation(int sid, Attribute<unsigned>* segPoints)
{
	Attribute<float> *arclength = dynamic_cast<Attribute<float>*>(_pStreamlines->streamlines[sid]->attributes["arclength"]);
	Attribute<float3> *coords = dynamic_cast<Attribute<float3>*>(_pStreamlines->streamlines[sid]->attributes["points"]);

	unsigned prev = 0;
	unsigned next;
	vector<unsigned> tmp;



	vector<unsigned> points;
	points.push_back(0);
	points.insert(points.end(), segPoints->data().begin(), segPoints->data().end());
	points.push_back(_pStreamlines->streamlines[sid]->numPts - 1);


	float threshold = 0;
	for (int i = 0; i < int(points.size()) - 1; i++) {
		for (int j = i + 1; j < points.size(); j++) 
			threshold += (*arclength)[points[j]] - (*arclength)[points[i]];
	}
	threshold = threshold / ((points.size() - 1) * points.size());
	
	
	
	vector<vector<unsigned>> groups;
	prev = 0;
	for (size_t i = 0; i < segPoints->size(); ) {
		unsigned cur = (*segPoints)[i];		
		if (((*arclength)[cur] - (*arclength)[prev]) > threshold) {

			tmp.clear();
			tmp.push_back(cur);
			size_t j = i + 1;
			while (j < segPoints->size()) {
				unsigned next = (*segPoints)[j];
				if (((*arclength)[cur] - (*arclength)[prev]) <= threshold) {
					tmp.push_back(next);
					j++;
				}
				else
					break;
			}
			groups.push_back(tmp);

			prev = cur;
			i = j;
		}
		else
			i++;
	}

	if (groups.empty()) {
		int id = segPoints->data().back();
		segPoints->data().clear();
		segPoints->push_back(id);
		return;
	}
	//assert(!groups.empty());

	if (arclength->back() - (*arclength)[(groups.back())[0]] <= threshold)
		groups.pop_back();


	tmp.clear();
	prev = 0;


	
	for (size_t i = 0; i < groups.size(); i++) {
		if (groups[i].size() > 1) {
			if (i == groups.size() - 1)
				next = _pStreamlines->streamlines[sid]->numPts - 1;
			else
				next = groups[i + 1][0];

			float max_diff = 0;
			float min_ratio = FLT_MAX;
			unsigned candidate;
			for (auto cut : groups[i]) {
				float ratio = MVEE::instance().computeBoundingEllipsoidVolRatio(sid, prev, cut, next);
				if (ratio < min_ratio) {
					min_ratio = ratio;
					candidate = cut;
				}

			}

			if ( min_ratio <= 0.8 )
				tmp.push_back(candidate);
		}
		else
			tmp.push_back(groups[i][0]);
		prev = tmp.back();
	}
	segPoints->data() = tmp;
}


