#include "segmenthistcalculator.h"
#include "streamlinepool.h"
#include <algorithm>
#include <iostream>

#define PI 3.1415927

void  SegmentHistogramCalculator::computeHistogram(int sid, int start, int end, vector<float>& hist)
{
	vector<float> hist1(_totalCurvBin, 0);
	computeCurvHistogram(sid, start, end, hist1);
	vector<float> hist2(_tiotalTorsBin, 0);
	computeTorsHistogram(sid, start, end, hist2);

	copy(hist1.begin(), hist1.end(), back_inserter(hist));
	copy(hist2.begin(), hist2.end(), back_inserter(hist));

}

void SegmentHistogramCalculator::computeCurvHistogram(int sid, int start, int end, vector<float>& hist)
{
	Attribute<float>* curv = dynamic_cast<Attribute<float>*>(_pStreamlines->streamlines[sid]->attributes["curvature"]);
	vector<float> work;
	for (int i = start; i < end; i++)
		work.push_back((*curv)[i]);

	computeHist_helper(work, hist, PI/float(_totalCurvBin));
}


void SegmentHistogramCalculator::computeTorsHistogram(int sid, int start, int end, vector<float>& hist)
{
	Attribute<float>* tors = dynamic_cast<Attribute<float>*>(_pStreamlines->streamlines[sid]->attributes["torsion"]);
	vector<float> work;
	for (int i = start; i < end; i++)
		work.push_back((*tors)[i]);

	computeHist_helper(work, hist, 2*PI/float(_tiotalTorsBin));
}

void SegmentHistogramCalculator::computeHist_helper(vector<float>& data, vector<float>& hist, float bucket_size)
{

	// compute histogram
	for (auto i : data)
		hist[int(floor(i/ bucket_size))]++;

	// normalize histogram
	for (auto& i : hist)
		i /= (float)data.size();
}