#include "streamlinecluster.h"
#include "streamlinepool.h"
#include "segmenthistcalculator.h"
#include "coarsesegmenter.h"

#include <algorithm>
#include <fstream>
#include <cassert>
#include <iostream>
#include <string>
#include <tuple>

#include <qdir.h>
#include <qprocess.h>
#include <qstringlist.h>

void StreamlineCluster::cluster()
{
	QString prefix("C:/Users/lyifei/work/FlowVis/bin/");
	QDir tmp_dir(prefix+"tmp");
	if (tmp_dir.exists())
		tmp_dir.removeRecursively();

	int n = _pStreamlines->streamlines.size();
	computeSimilarityMatrix();
	ofstream outfile(prefix.toStdString() + "similarities.txt");
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < n; j++) {
			if ( i != j )
				outfile << i+1 << " " << j+1 << " " << _sij[i*n + j] << endl;
		}
	}
	outfile.close();

	string cmd = prefix.toStdString() + "apcluster.exe " + prefix.toStdString() + "similarities.txt " + prefix.toStdString() + "pref.txt " + prefix.toStdString() + "tmp";
	int retCode = system(cmd.c_str());
	assert(retCode >= 0);

	clusters.clear();
	ifstream infile(prefix.toStdString() + "tmp/idx.txt");
	int center, counter = 0;
	while (infile >> center) 
		clusters[center - 1].push_back(counter++);
	infile.close();

	cout << "# streamline clusters: " << clusters.size() << endl;
}

void StreamlineCluster::clusterSegments()
{
	// collect all segments
	segments.clear();

	for (size_t sid = 0; sid < _pStreamlines->streamlines.size(); sid++) {
		Attribute<unsigned> *segpoints = dynamic_cast<Attribute<unsigned>*>(_pStreamlines->streamlines[sid]->attributes["segmentation_points"]);
		if (segpoints->size() > 0) {
			unsigned prev = 0;
			for (size_t i = 0; i < segpoints->size(); i++) {
				unsigned cur = (*segpoints)[i];
				segments.emplace_back(sid, prev, cur);
				prev = cur;
			}
			segments.emplace_back(sid, prev, _pStreamlines->streamlines[sid]->numPts - 1);
		}
		else
			segments.emplace_back(sid, 0, _pStreamlines->streamlines[sid]->numPts - 1);
	}

	QString prefix("C:/Users/lyifei/work/FlowVis/bin/");
	QDir tmp_dir(prefix + "tmp");
	if (tmp_dir.exists())
		tmp_dir.removeRecursively();

	int n = segments.size();
	computeSegmentSimilarityMatrix(segments);
	ofstream outfile(prefix.toStdString() + "similarities.txt");
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < n; j++) {
			if (i != j)
				outfile << i + 1 << " " << j + 1 << " " << _sij[i*n + j] << endl;
		}
	}
	outfile.close();

	string cmd = prefix.toStdString() + "apcluster.exe " + prefix.toStdString() + "similarities.txt " + prefix.toStdString() + "pref.txt " + prefix.toStdString() + "tmp";
	int retCode = system(cmd.c_str());
	assert(retCode >= 0);


	segmentClusters.clear();
	ifstream infile(prefix.toStdString() + "tmp/idx.txt");
	int center, counter = 0;
	while (infile >> center)
		segmentClusters[center - 1].push_back(counter++);
	infile.close();


}

void StreamlineCluster::changeSegmentClusters(int val) {
	size_t n = segments.size();
	QString prefix("C:/Users/lyifei/work/FlowVis/bin/");

	if (recompute_similarity) {
		computeSegmentSimilarityMatrix(segments);
		ofstream outfile(prefix.toStdString() + "similarities.txt");
		for (int i = 0; i < n; i++) {
			for (int j = 0; j < n; j++) {
				if (i != j)
					outfile << i + 1 << " " << j + 1 << " " << _sij[i*n + j] << endl;
			}
		}
		outfile.close();
		recompute_similarity = false;
	}

	_preference = _minSimilarity + (float)val / 100 * (_medianSimilarity - _minSimilarity);
	

	ofstream outfile("C:/Users/lyifei/work/FlowVis/bin/pref.txt");
	for (size_t i = 0; i < n; i++)
		outfile << _preference << endl;
	outfile.close();

	

	string cmd = prefix.toStdString() + "apcluster.exe " + prefix.toStdString() + "similarities.txt " + prefix.toStdString() + "pref.txt " + prefix.toStdString() + "tmp";
	int retCode = system(cmd.c_str());
	assert(retCode >= 0);


	segmentClusters.clear();
	ifstream infile(prefix.toStdString() + "tmp/idx.txt");
	int center, counter = 0;
	while (infile >> center)
		segmentClusters[center - 1].push_back(counter++);
	infile.close();
}

void StreamlineCluster::computeSimilarityMatrix()
{
	size_t n = _pStreamlines->streamlines.size();
	// compute histogram
	vector<vector<float>> histograms(n);
	for (size_t i = 0; i < _pStreamlines->streamlines.size(); i++) 
		SegmentHistogramCalculator::instance().computeHistogram(i, 0, _pStreamlines->streamlines[i]->numPts - 1, histograms[i]);
	


	if (_sij) {
		delete[] _sij;
		_sij = nullptr;
	}

	
	_sij = new float[n*n];
	vector<float> tmp;
	for (size_t i = 0; i < n; i++) {
		for (size_t j = i + 1; j < n; j++) {
			float dist = -CoarseSegmenter::instance().segment_dist(histograms[i], histograms[j]);
			_sij[i*n + j] = _sij[j*n + i] = dist;
			tmp.push_back(dist);
		}
	}

	sort(tmp.begin(), tmp.end());
	_minSimilarity = tmp[0];
	_medianSimilarity = tmp[tmp.size() / 2];

	ofstream outfile("C:/Users/lyifei/work/FlowVis/bin/pref.txt");
	for (size_t i = 0; i < n; i++)
		outfile << _medianSimilarity << endl;
		//outfile << _minSimilarity << endl;
	outfile.close();
}

void StreamlineCluster::computeSegmentSimilarityMatrix(const vector<tuple<unsigned, unsigned, unsigned>>& segments)
{
	size_t n = segments.size();
	// compute histogram
	vector<vector<float>> histograms(n);
	for (size_t i = 0; i < n; i++) {
		int sid = std::get<0>(segments[i]);
		int start = std::get<1>(segments[i]);
		int end = std::get<2>(segments[i]);
		SegmentHistogramCalculator::instance().computeHistogram(sid, start, end, histograms[i]);
	}



	if (_sij) {
		delete[] _sij;
		_sij = nullptr;
	}


	_sij = new float[n*n];
	vector<float> tmp;
	for (size_t i = 0; i < n; i++) {
		for (size_t j = i + 1; j < n; j++) {
			float dist = -CoarseSegmenter::instance().segment_dist(histograms[i], histograms[j]);
			_sij[i*n + j] = _sij[j*n + i] = dist;
			tmp.push_back(dist);
		}
	}

	sort(tmp.begin(), tmp.end());
	_minSimilarity = tmp[0];
	_medianSimilarity = tmp[tmp.size() / 2];

	if (_preference == 0)
		_preference = _minSimilarity;

	ofstream outfile("C:/Users/lyifei/work/FlowVis/bin/pref.txt");

	for (size_t i = 0; i < n; i++)
		outfile << _preference << endl;
	outfile.close();

}