#include "tortuositycalculator.h"
#include "streamlinepool.h"
#include "vector_types.h"
#include <algorithm>
#include <cassert>

float TortuosityCalculator::computeTortuosityRatio(int sid, int center, int nhs)
{
	Streamline* s = _pStreamlines->streamlines[sid];
	Attribute<float> *arclength = dynamic_cast<Attribute<float>*>(s->attributes["arclength"]);
	Attribute<float3> *points = dynamic_cast<Attribute<float3>*>(s->attributes["points"]);

	if (center - nhs < 0)
		nhs = center;
	if (center + nhs > int(s->numPts) - 1)
		nhs = int(s->numPts) - 1 - center;

	int left = center - nhs;
	int right = center + nhs;

	float arclen1 = (*arclength)[center] - (*arclength)[left];
	float arclen2 = (*arclength)[right] - (*arclength)[center];

	float dist1 = length((*points)[center] - (*points)[left]);
	float dist2 = length((*points)[right] - (*points)[center]);

	assert(dist1 != 0);
	assert(dist2 != 0);

	float tort1 = arclen1 / dist1;
	float tort2 = arclen2 / dist2;

	if (tort1 > tort2)
		swap(tort1, tort2);

	return tort1 / tort2;
}