#ifndef TRAININGDATA_H
#define TRAININGDATA_H

#include <vector>
#include <unordered_map>
#include <string>
#include <set>
#include <unordered_set>
#include <memory>

#include "vector_types.h"
#include "svm.h"
using namespace std;

class StreamlinePool;


class Trainer{
public:
	static Trainer& instance() {
		static Trainer data;
		return data;
	}

	void reset() {
		_currentSid = -1;
		_segmentPoints.clear();
	}

	void setStreamlinePool(shared_ptr<StreamlinePool> stl) { _pStreamlines = stl;  }

	// call this function to train the classifer
	void train();

	void removeLastSegPoint() { 
		_segPointExamples.pop_back();  
		_segmentPoints.erase(_lastSegPoint); 
	}

	bool compute_point_features(int sid, int id, vector<float>& feature);

	void add_seg_point_example(int id, bool seg_point);

	void add_seg_point_range(int id, int left, int right);

	struct svm_model* get_seg_point_classifier() const { return _segPointModel;  }

	void clearTrainingData() { _segPointExamples.clear(); }

	void reTrain();

private:
	Trainer();
	~Trainer();
	void addNegativeSegPointTrainingData(const vector<int>&);

	void genernateNHS(vector<int>&, int sid, int center);

	void compute_segpoint_classifier();

	struct svm_model* linear_SVM(const vector<vector<float>>&, bool weighting);
	struct svm_model* rbf_SVM(const vector<vector<float>>&, bool weighting, double& auc);

	float segment_dist(const vector<float>& hist1, const vector<float>& hist2);

private:
	struct ExampleRecord {
		vector<unsigned>	seg_point_ids;
		vector<unsigned>    non_seg_point_ids;
	};

	unordered_map<int, ExampleRecord>	_exampleRecords;

	int									_currentSid;			///< current streamline picked for traning
	set<int>							_segmentPoints;
	unordered_map<int, pair<int, int>>	_segPointRange;
	int									_lastSegPoint;

	shared_ptr<StreamlinePool>			_pStreamlines;

	struct svm_model					*_segPointModel;

	vector<vector<float>>				_segPointExamples;

	int									_accuracy;

	bool								_trainingFinished;

	int									_numberOfNeighborhoods;
	int									_startPercentage;
};

#endif