#ifndef VECTOR_TYPES_H
#define VECTOR_TYPES_H

#include <cmath>

struct float3
{
    float x, y, z;
};

struct float4
{
	float x, y, z, w;
};

inline float3 make_float3(float a, float b, float c)
{
    float3 res;
    res.x = a; res.y = b; res.z = c;
    return res;
}

inline float4 make_float4(float a, float b, float c, float d)
{
	float4 res;
	res.x = a; res.y = b; res.z = c; res.w = d;
	return res;
}


inline float3 operator-(float3 &a)
{
    return make_float3(-a.x, -a.y, -a.z);
}

inline float3 operator-(float3 a, float3 b)
{
    return make_float3(a.x-b.x, a.y-b.y, a.z-b.z);
}

inline float3 operator+(float3 a, float3 b)
{
    return make_float3(a.x + b.x, a.y + b.y, a.z + b.z);
}

inline float3 operator*(float3 a, float s)
{
    return make_float3(a.x*s, a.y*s, a.z*s);
}

inline float3 operator*(float s, float3 a)
{
    return make_float3(a.x*s, a.y*s, a.z*s);
}

inline float3 cross(float3 a, float3 b)
{
    return make_float3(a.y*b.z - a.z*b.y, a.z*b.x - a.x*b.z, a.x*b.y - a.y*b.x);
}

inline float dot(float3 a, float3 b)
{
    return a.x * b.x + a.y * b.y + a.z * b.z;
}

inline float3 normalize(float3 v)
{
    float invLen = 1.0f / sqrtf(dot(v, v));
    return v * invLen;
}

inline float length(float3 v)
{
    return sqrtf(dot(v, v));
}


#endif // VECTOR_TYPES_H
